package es.verdnatura.db

import android.content.Context
import androidx.room.*
import es.verdnatura.presentation.view.feature.delivery.model.ExpeditionInfoLoadUnload
import es.verdnatura.presentation.view.feature.delivery.model.ExpeditionInfoPending


// Clase que representa la base de datos
@Database(entities = [ExpeditionInfoLoadUnload::class,ExpeditionInfoPending::class], version = 1)
abstract class ExpedicionDatabase : RoomDatabase() {
    abstract fun expedicionDao(): ExpedicionDao

    companion object {
        @Volatile
        private var INSTANCE: ExpedicionDatabase? = null

        fun getInstance(context: Context): ExpedicionDatabase {
            return INSTANCE ?: synchronized(this) {
                val instance = Room.databaseBuilder(
                    context.applicationContext,
                    ExpedicionDatabase::class.java,
                    "expedition_database"
                )
                    .fallbackToDestructiveMigration()
                    .build()
                INSTANCE = instance
                instance
            }
        }
    }
}

// Interfaz que define los métodos de acceso a datos para Expedicion
@Dao
interface ExpedicionDao {
    @Query("SELECT * FROM expedition e WHERE e.routeFk = :parametro1")
    suspend fun getAll(parametro1:Int): List<ExpeditionInfoLoadUnload>

  /*  @Query("SELECT * FROM expeditionPending")
    suspend fun getAllPending(): List<ExpeditionInfoLoadUnload>*/

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insert(expedition:List<ExpeditionInfoLoadUnload>)

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertPending(expeditionPending:MutableList<ExpeditionInfoPending>)

    @Delete
    fun delete(expedition: ExpeditionInfoLoadUnload)
}

