package es.verdnatura.domain

import android.content.Context
import es.verdnatura.R
import es.verdnatura.presentation.base.nameofFunction
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


//Se crea interceptor para así generalizar las respuestas y llamadas,
//si hay respuesta de fallo 555 se llama onError que lanza un error para que se haga el catch() directamente,
//si no queremos que haga el catch tenemos que reescribir la llamada onError en la clase donde se llama o incluso
// lanzar un throw error de nuevo para que vuelva al catch y lo ejecute.
//-> sería válido en el caso que el catch fuera lo general para todos ( por ejemplo un Toast y después realizáramos
//otras acciones. Se reescribiria el método onError en LoginViewModel como:
// do_actions-> Haz acciones del fallo
// throw e -> De esta manera se consigue llegar al catch. El error se propaga a la llamada superior.

//Falta pasarle al Callback el tipo que queramos y poder lanzar el toast


abstract class SilexCallback<T>(val context: Context) : Callback<T> {

    override fun onResponse(call: Call<T>, response: Response<T>) {
        if (response.isSuccessful) {
            onSuccess(response)
        } else {
            try {
                if (response.code() == 555) {
                    var message = JSONObject(response.message()).getString("Message")
                    onError(Error(message))
                } else {
                    onError(Error(response.message().toString()))
                }
            } catch (t: Throwable) {
                defaultErrorHandler(t)
            }
        }
    }

    override fun onFailure(call: Call<T>, t: Throwable) {
        try {
            onError(t)
        } catch (t: Throwable) {
            defaultErrorHandler(t)
        }
    }

    private fun defaultErrorHandler(t: Throwable) {
        (nameofFunction((this)) + t.message).toast(context)
        //(context as MobileApplication).playSoundIsOK(false)
    }

    open fun onSuccess(response: Response<T>) {
        // (nameofFunction((this)) + context.getString(R.string.operationSuccessful)).toast(context)
        //(context as MobileApplication).playSoundIsOK(true)
    }

    open fun onError(t: Throwable) {
        throw t
    }


}