package es.verdnatura.presentation.view.feature.articulo.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.common.itemBarCodeSalix
import es.verdnatura.presentation.common.itemPackingTypeSalix
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemPackingTypeList
import es.verdnatura.presentation.view.feature.articulo.model.ItemProposal
import es.verdnatura.presentation.view.feature.articulo.model.ItemProposalList
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Response
import java.io.File
import java.util.*
import kotlin.collections.ArrayList

class ItemCardViewModel(var context: Context) : BaseViewModel(context) {

    private val _itemcard by lazy { MutableLiveData<ItemCardVO>() }
    val itemcard: LiveData<ItemCardVO>
        get() = _itemcard

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _itemspackinglist by lazy { MutableLiveData<ItemPackingTypeList>() }
    val itemspackinglist: LiveData<ItemPackingTypeList>
        get() = _itemspackinglist


    private val _itemProposallist by lazy { MutableLiveData<ItemProposalList>() }
    val itemProposallist: LiveData<ItemProposalList>
        get() = _itemProposallist


    fun getItemCard(
        itemFk: Int,
        warehouseFk: Int,

        ) {
        silex.getItemCard(itemFk, warehouseFk)
            .enqueue(object : SilexCallback<ItemCardVO>(context) {
                override fun onError(t: Throwable) {
                    _itemcard.value = ItemCardVO(
                        0,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!),
                        stems = "", itemCost = ""
                    )

                }

                override fun onSuccess(response: Response<ItemCardVO>) {
                    if (response.body() != null) {

                        _itemcard.value = response.body()!!
                    } else {
                        _itemcard.value = ItemCardVO(
                            0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            ),
                            stems = "",
                            itemCost = ""
                        )
                    }
                }

            })
    }

    fun item_getSimilar(
        itemFk: Int,
        warehouseFk: Int,
        vShipped: String,
        vIsShowedByType: Boolean,

        ) {
        silex.item_getSimilar(
            itemFk,
            warehouseFk,
            vShipped,
            vIsShowedByType
        ).enqueue(object : SilexCallback<List<ItemProposal>>(context) {
            override fun onError(t: Throwable) {
                val listError: ArrayList<ItemProposal> = ArrayList()
                listError.add(
                    ItemProposal(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _itemProposallist.value = ItemProposalList(listError)
            }

            override fun onSuccess(response: Response<List<ItemProposal>>) {

                if (response.body() != null) {
                    _itemProposallist.value = response.body()?.let {
                        ItemProposalList(it)
                    }

                } else {
                    val listError: ArrayList<ItemProposal> = ArrayList()
                    listError.add(
                        ItemProposal(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _itemProposallist.value = ItemProposalList(listError)

                }
            }

        })
    }

    fun itemTrash(
        itemFk: Int,
        warehouseFk: Int,
        newValue: Int,
        isTrash: Boolean
    ) {
        silex.itemTrash(itemFk, warehouseFk, newValue, isTrash)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

    fun updateGrouping(
        itemFk: Int,
        value: Int,
        warehouseFk: Int

    ) {
        silex.updateGrouping(itemFk, value, warehouseFk)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    super.onError(t)
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

    fun updatePacking(
        itemFk: Int,
        value: Int,
        warehouseFk: Int

    ) {
        silex.updatePacking(itemFk, value, warehouseFk)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {

                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())
                }

            })
    }

    fun updatePackingTypeSalix(itemFk: String, itemPackingType: itemPackingTypeSalix) {
        salix.update_ItemPackingType(itemFk, itemPackingType)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.body()!!.toString())

                }

            })
    }

    fun item_saveStems(
        itemFk: Int,
        value: Int

    ) {
        silex.item_saveStems(itemFk, value)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

    fun item_saveReference(
        itemFk: Int,
        value: Int

    ) {
        silex.item_saveReference(itemFk, value)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value = ResponseItemVO(isError = false)
                }

            })
    }

    fun barcodesEdit(
        itemFk: Int,
        value: String,
        delete: Boolean
    ) {
        silex.barcodes_edit(value, itemFk, delete)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value = ResponseItemVO(
                        isError = false,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            response.message()
                        )
                    )
                }

            })
    }


    fun barcodesEditSalix(itemFk: Int, barCodeValue: String) {
        salix.barcodes_edit(
            itemBarCodeSalix(itemFk, barCodeValue)
        )
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message()!!)
                }

            })
    }



    private val _responseinsert by lazy { MutableLiveData<ResponseItemVO>() }
    val responseinsert: LiveData<ResponseItemVO>
        get() = _responseinsert


    fun uploadPhoto(
        urlImage:String,
        itemFK:Int,
        file: File = File("$urlImage"),
        fileRequestBody: RequestBody = RequestBody.create(MediaType.parse("image/jpeg"), file),
        filePart: MultipartBody.Part = MultipartBody.Part.createFormData("file", file.name, fileRequestBody),
            ) {
        salix.uploadImage("catalog",itemFK,filePart)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _responseinsert.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _responseinsert.value = ResponseItemVO(
                        isError = false

                    )
                }

            })
    }



}
