package es.verdnatura.presentation.view.feature.delivery.viewmodels


import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.delivery.model.*
import retrofit2.Response

class DeliveryViewModel(val context: Context) : BaseViewModel(context) {

    private val _expeditionInfoList by lazy { MutableLiveData<ExpeditionInfoList>() }
    val expeditionInfoList: LiveData<ExpeditionInfoList> = _expeditionInfoList

    private val _expeditionInfoLoadUnloadList by lazy { MutableLiveData<ExpeditionList>() }
    val expeditionInfoLoadUnloadList: LiveData<ExpeditionList> = _expeditionInfoLoadUnloadList

    private val _expeditionInfoSummary by lazy { MutableLiveData<ExpeditionSummaryList>() }
    val expeditionInfoSummary: LiveData<ExpeditionSummaryList> = _expeditionInfoSummary

    private val _routeInfoList by lazy { MutableLiveData<RouteDeliveryList>() }
    val routeInfoList: LiveData<RouteDeliveryList> = _routeInfoList

    private val _freelanceInfoList by lazy { MutableLiveData<FreeLanceDeliveryInfoList>() }
    val freelanceInfoList: LiveData<FreeLanceDeliveryInfoList> = _freelanceInfoList

    private val _companyInfoList by lazy { MutableLiveData<FreeLanceDeliveryInfoList>() }
    val companyInfoList: LiveData<FreeLanceDeliveryInfoList> = _companyInfoList

    private val _responseStateAdd by lazy { MutableLiveData<ResponseItemVO>() }
    val responseStateAdd: LiveData<ResponseItemVO>
        get() = _responseStateAdd

    private val _expeditionList by lazy { MutableLiveData<ExpeditionList>() }
    val expeditionList: LiveData<ExpeditionList> = _expeditionList


    fun getInfoCompany(
        userId: Int
    ) {

        silex.getInfoCompany(userId)
            .enqueue(object : SilexCallback<List<DeliveryInfo>>(context) {
                override fun onSuccess(response: Response<List<DeliveryInfo>>) {

                    if (response.body() != null) {
                        _companyInfoList.value =
                            response.body()?.let { FreeLanceDeliveryInfoList(it) }
                    } else {
                        _companyInfoList.value = FreeLanceDeliveryInfoList()
                    }
                }

                override fun onError(t: Throwable) {
                    _companyInfoList.value = FreeLanceDeliveryInfoList()
                    super.onError(t)

                }
            })
    }

    fun expedition_getLog(
        expedition: Int
    ) {

        silex.expedition_getLog(expedition)
            .enqueue(object : SilexCallback<List<ExpeditionInfoLog>>(context) {

                override fun onSuccess(response: Response<List<ExpeditionInfoLog>>) {

                    if (response.body() != null) {
                        _expeditionInfoList.value =
                            response.body()?.let { ExpeditionInfoList(it) }
                    } else {
                        _expeditionInfoList.value = ExpeditionInfoList()
                    }
                }

                override fun onError(t: Throwable) {
                    _expeditionInfoList.value = ExpeditionInfoList()
                    super.onError(t)

                }
            })
    }

    fun getExpeditionFromRoute(
        route: Int
    ) {

        silex.getExpeditionFromRoute(route)
            .enqueue(object : SilexCallback<List<ExpeditionInfoLoadUnload>>(context) {

                override fun onSuccess(response: Response<List<ExpeditionInfoLoadUnload>>) {

                    if (response.body() != null) {
                        _expeditionInfoLoadUnloadList.value =
                            response.body()?.let { ExpeditionList(it) }
                    } else {
                        _expeditionInfoLoadUnloadList.value = ExpeditionList()
                    }
                }

                override fun onError(t: Throwable) {
                    _expeditionInfoLoadUnloadList.value = ExpeditionList()
                    super.onError(t)

                }
            })
    }


    fun get_routesFromExpedition(
        expedition: Int
    ) {

        silex.get_routesFromExpedition(expedition)
            .enqueue(object : SilexCallback<List<RouteDelivery>>(context) {

                override fun onSuccess(response: Response<List<RouteDelivery>>) {
                    if (response.body() != null) {
                        _routeInfoList.value =
                            response.body()?.let { RouteDeliveryList(it) }

                    }
                }

                override fun onError(t: Throwable) {
                    _routeInfoList.value = RouteDeliveryList()
                    super.onError(t)
                }
            })

    }

    fun expeditionState_add(expeditions: Any?, state: String) {

        silex.expeditionState_add(expeditions, state)
            .enqueue(object : SilexCallback<Void>(context) {
                override fun onSuccess(response: Response<Void>) {
                    _responseStateAdd.value = ResponseItemVO()
                }

                override fun onError(t: Throwable) {
                    super.onError(t)
                    _responseStateAdd.value = ResponseItemVO()
                }
            })
    }


    fun getInfoFreelance(
        userId: Int
    ) {
        silex.getInfoFreelance(userId)
            .enqueue(object : SilexCallback<List<DeliveryInfo>>(context) {

                override fun onSuccess(response: Response<List<DeliveryInfo>>) {

                    if (response.body() != null) {
                        _freelanceInfoList.value =
                            response.body()?.let { FreeLanceDeliveryInfoList(it) }
                    } else {
                        _freelanceInfoList.value = FreeLanceDeliveryInfoList()
                    }
                }

                override fun onError(t: Throwable) {
                    _freelanceInfoList.value = FreeLanceDeliveryInfoList()
                    super.onError(t)

                }
            })
    }

    fun route_getExpeditionSummary(
        route: Int
    ) {
        silex.route_getExpeditionSummary(route)
            .enqueue(object : SilexCallback<List<ExpeditionInfoSummary>>(context) {

            override fun onSuccess(response: Response<List<ExpeditionInfoSummary>>) {

                if (response.body() != null) {
                    _expeditionInfoSummary.value =
                        response.body()?.let { ExpeditionSummaryList(it) }
                } else {
                    _expeditionInfoSummary.value = ExpeditionSummaryList()
                }
            }

            override fun onError(t: Throwable) {
                _expeditionInfoSummary.value = ExpeditionSummaryList()
                super.onError(t)

            }
        })
    }


}


