package es.verdnatura.presentation.view.feature.inventario.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.calidad.model.BuyerListVO
import es.verdnatura.presentation.view.feature.calidad.model.BuyerVO
import es.verdnatura.presentation.view.feature.inventario.model.InventaryListVO
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import es.verdnatura.presentation.view.feature.workermistake.model.*
import retrofit2.Response

class InventaryViewModel(val context: Context) : BaseViewModel(context) {

    private val _inventaryList by lazy { MutableLiveData<InventaryListVO>() }
    val inventaryList: LiveData<InventaryListVO>
        get() = _inventaryList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    private val _buyersList by lazy { MutableLiveData<BuyerListVO>() }
    val buyersList: LiveData<BuyerListVO>
        get() = _buyersList

    private val _mistakeDepartmentList by lazy { MutableLiveData<departmentMistakeList>() }
    val mistakeDepartmentList: LiveData<departmentMistakeList>
        get() = _mistakeDepartmentList

    private val _workerFromMistakeList by lazy { MutableLiveData<workerFromMistakeList>() }
    val workerFromMistakeList: LiveData<workerFromMistakeList>
        get() = _workerFromMistakeList

    private val _mistakeWorkerList by lazy { MutableLiveData<workerMistakeTypeList>() }
    val mistakeWorkerList: LiveData<workerMistakeTypeList>
        get() = _mistakeWorkerList

    private val _responseAddMistake by lazy { MutableLiveData<ResponseItemVO>() }
    val responseAddMistake: LiveData<ResponseItemVO>
        get() = _responseAddMistake

    //sergio para obtener comprador
    fun itemShelvingBuyerGet() {
       silex.itemShelvingBuyerGet().enqueue(object :
            SilexCallback<List<BuyerVO>>(context) {
           override fun onError(t: Throwable) {
                val listError: ArrayList<BuyerVO> = ArrayList()
                listError.add(
                    BuyerVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _buyersList.value = BuyerListVO(listError)
            }

           override fun onSuccess(response: Response<List<BuyerVO>>) {
                if (response.body() != null) {
                    _buyersList.value = response.body()?.let { BuyerListVO(it) }
                } else {
                    val listError: ArrayList<BuyerVO> = ArrayList()
                    listError.add(
                        BuyerVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _buyersList.value = BuyerListVO(listError)
                }
            }

        })
    }


    val loadInventaryList = Transformations.map(_inventaryList) { Event(it) }

    fun getInventory(buyerFk: Int, warehouseFk: Int) {
       silex.itemShelvingRadarFilterBuyer(buyerFk, warehouseFk)
            .enqueue(object :
                SilexCallback<List<ItemInventaryVO>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<ItemInventaryVO> = ArrayList()
                    listError.add(
                        ItemInventaryVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _inventaryList.value = InventaryListVO(listError)
                }

                override fun onSuccess(response: Response<List<ItemInventaryVO>>) {
                    if (response.body() != null) {
                        _inventaryList.value = response.body()?.let { InventaryListVO(it) }
                    } else {
                        val listError: ArrayList<ItemInventaryVO> = ArrayList()
                        listError.add(
                            ItemInventaryVO(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _inventaryList.value = InventaryListVO(listError)

                    }
                }

            })
    }


    //sergio: se modifica por CAU a itemTrash
    fun itemTrash(
        itemFk: Int,
        warehouseFk: Int,
        newValue: Int,
        isTrash: Boolean
    ) {


       silex.itemTrash(itemFk, warehouseFk, newValue, isTrash)
            .enqueue(object : SilexCallback<Any>(context) {

                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.message())
                }

            })
    }

    //En test encara,no provat en Salix
   /* fun department_getHasMistakeSalix(token: String) {

        getWorkerMistakeUserCase.department_getHasMistakeSalix(token)

            .enqueue(object : Callback<List<DepartmentMistake>> {
                override fun onFailure(call: Call<List<DepartmentMistake>>, t: Throwable) {
                    val listError: ArrayList<DepartmentMistake> = ArrayList()
                    listError.add(
                        DepartmentMistake(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _mistakeDepartmentList.value = departmentMistakeList(listError)
                }

                override fun onResponse(
                    call: Call<List<DepartmentMistake>>,
                    response: Response<List<DepartmentMistake>>
                ) {
                    if (response.body() != null) {
                        _mistakeDepartmentList.value =
                            response.body()?.let { departmentMistakeList(it) }
                    } else {
                        val listError: ArrayList<DepartmentMistake> = ArrayList()
                        listError.add(DepartmentMistake(0, ""))
                        _mistakeDepartmentList.value = departmentMistakeList(listError)
                    }
                }

            })
    }*/

    fun department_getHasMistake() {

       silex.department_getHasMistake()
            .enqueue(object : SilexCallback<List<DepartmentMistake>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<DepartmentMistake> = ArrayList()
                    listError.add(
                        DepartmentMistake(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _mistakeDepartmentList.value = departmentMistakeList(listError)
                }

                override fun onSuccess(response: Response<List<DepartmentMistake>>) {
                    if (response.body() != null) {
                        _mistakeDepartmentList.value =
                            response.body()?.let { departmentMistakeList(it) }
                    } else {
                        val listError: ArrayList<DepartmentMistake> = ArrayList()
                        listError.add(DepartmentMistake(0, ""))
                        _mistakeDepartmentList.value = departmentMistakeList(listError)
                    }
                }

            })
    }

    fun worker_getFromHasMistake( departmentFk: Int) {
       silex.worker_getFromHasMistake(departmentFk)
            .enqueue(object :
                SilexCallback<List<WorkerFromMistake>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<WorkerFromMistake> = ArrayList()
                    listError.add(
                        WorkerFromMistake(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _workerFromMistakeList.value = workerFromMistakeList(listError)
                }

                override fun onSuccess(response: Response<List<WorkerFromMistake>>) {
                    if (response.body() != null) {

                        _workerFromMistakeList.value =
                            response.body()?.let { workerFromMistakeList(it) }
                    } else {
                        val listError: ArrayList<WorkerFromMistake> = ArrayList()
                        listError.add(
                            WorkerFromMistake(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _workerFromMistakeList.value = workerFromMistakeList(listError)
                    }
                }

            })
    }

    fun workerMistakeType_get() {
       silex.workerMistakeType_get()
            .enqueue(object : SilexCallback<List<MistakeWorkerType>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<MistakeWorkerType> = ArrayList()
                    listError.add(
                        MistakeWorkerType(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            ),
                            id = "0",
                            description = ""
                        )
                    )
                    _mistakeWorkerList.value = workerMistakeTypeList(listError)
                }

                override fun onSuccess(response: Response<List<MistakeWorkerType>>) {
                    if (response.body() != null) {
                        _mistakeWorkerList.value =
                            response.body()?.let { workerMistakeTypeList(it) }
                    } else {
                        val listError: ArrayList<MistakeWorkerType> = ArrayList()
                        listError.add(
                            MistakeWorkerType(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                ),
                                id = "0",
                                description = ""
                            )
                        )
                        _mistakeWorkerList.value = workerMistakeTypeList(listError)
                    }
                }

            })
    }


    fun workerMistake_Add(

        vUserFk: Int,
        vTypeFk: String
    ) {
       silex.workerMistake_Add(vUserFk, vTypeFk)
            .enqueue(object :
                SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _responseAddMistake.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                        // sergio:refactor:solo debería haber un mensaje no un response o errorMessage
                        _responseAddMistake.value =
                            ResponseItemVO(
                                isError = false,
                                response = response.message()
                            )

                }
            })
    }

    fun expeditionMistake_add(
        vExpeditionFk: Int,
        vTypeFk: String
    ) {
       silex.expeditionMistake_add(vExpeditionFk, vTypeFk)
            .enqueue(object :
                SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _responseAddMistake.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {

                    // sergio:refactor:solo debería haber un mensaje no un response o errorMessage
                        _responseAddMistake.value =
                            ResponseItemVO(
                                isError = false,
                                response = response.message()
                            )
                }
            })
    }

    fun expeditionMistakeType_get() {
       silex.expeditionMistakeType_get()
            .enqueue(object : SilexCallback<List<MistakeWorkerType>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<MistakeWorkerType> = ArrayList()
                    listError.add(
                        MistakeWorkerType(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            ),
                            id = "0",
                            description = ""
                        )
                    )
                    _mistakeWorkerList.value = workerMistakeTypeList(listError)
                }

                override fun onSuccess(response: Response<List<MistakeWorkerType>>) {

                    if (response.body() != null) {
                        _mistakeWorkerList.value =
                            response.body()?.let { workerMistakeTypeList(it) }
                    } else {
                        val listError: ArrayList<MistakeWorkerType> = ArrayList()
                        listError.add(
                            MistakeWorkerType(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                ),
                                id = "0",
                                description = ""
                            )
                        )
                        _mistakeWorkerList.value = workerMistakeTypeList(listError)
                    }
                }

            })
    }
}
